package com.xweld.services.cache.infinispan.redhat;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Future;

import org.infinispan.Cache;
import org.infinispan.distexec.DefaultExecutorService;
import org.infinispan.distexec.DistributedCallable;
import org.infinispan.distexec.DistributedExecutorService;
import org.infinispan.manager.CacheManager;
import org.infinispan.tree.Fqn;
import org.infinispan.tree.Node;
import org.infinispan.tree.TreeCache;
//import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;




public class TestMultiWriter implements Serializable
{
  private static final long   serialVersionUID                 = -7108137415096982449L;

  private static final String SOURCE_CACHE_SIGNATURE_NAME      = "Source";
  private static final String DESTINATION_CACHE_SIGNATURE_NAME = "Destination";

  private static final String SOURCE_CHILD_NODE_PREFIX         = "ChildNode";
  private static final int    NUM_SOURCE_NODES                 = 500;

  private static final Logger log                              = LoggerFactory
                                                                   .getLogger(TestMultiWriter.class);

//  private final String        dataSourceCacheManagerName ;
  
  public TestMultiWriter()
  {
    
  }

//  public MultiWriterTest(final String dataSourceCacheManagerName)
//  {
////    this.dataSourceCacheManagerName = dataSourceCacheManagerName;
//    
//  }

  protected CacheManager getCacheManager()
  {
    return TestHelper.getInstance().getCacheManager();
  }

  public static void main(String [] args)
  {
	 TestMultiWriter tmw = new TestMultiWriter();
	 tmw.runTest();
  }
  public void runTest()
  {
    try
    {
      log.info("runTest starting up");
      TestHelper.getInstance();
      
      final ITreeCacheSignatureTest sourceSignature = new MWTestCacheSignature(
          SOURCE_CACHE_SIGNATURE_NAME);
      final Cache<Object, Object> sourceCache = TestHelper.getInstance().createIfAbsent(sourceSignature);

      final ITreeCacheSignatureTest destinationSignature = new MWTestCacheSignature(
          DESTINATION_CACHE_SIGNATURE_NAME);
      final Cache<Object, Object> destinationCache = TestHelper.getInstance().createIfAbsent(destinationSignature);

      final List<Fqn> sourceChildPaths = populateSourceCache(sourceCache);

      runDistributedCallable(sourceCache, destinationCache, sourceChildPaths);

      log.info("runTest succeeded, removing caches");

      getCacheManager().removeCache(sourceCache.getName());
      getCacheManager().removeCache(destinationCache.getName());

      log.info("runTest complete");
    } catch (Exception ex)
    {
      log.error("runTest failed", ex);
    }
  }

  private List<Fqn> populateSourceCache(
      final Cache<Object, Object> sourceCache)
  {
    final List<Fqn> childPaths = new ArrayList<Fqn>(
        NUM_SOURCE_NODES);

    final TreeCache<Object, Object> treeCache = TestHelper.getInstance().getTreeCache(sourceCache);
    final Node<Object, Object> rootNode = treeCache.getRoot();

    for (int i = 0; i < NUM_SOURCE_NODES; i++)
    {
      final String childNodeName = SOURCE_CHILD_NODE_PREFIX
          + Integer.toString(i);
      final Fqn childPath = Fqn.fromString(childNodeName);
      rootNode.addChild(childPath);
      childPaths.add(childPath);
    }
    return childPaths;
  }

  private void runDistributedCallable(final Cache<Object, Object> sourceCache,
      final Cache<Object, Object> destinationCache,
      final List<Fqn> sourceChildPaths) throws Exception
  {
    try
    {
      final String sourceCacheName = sourceCache.getName();
      final String destinationCacheName = destinationCache.getName();

      final MWTestDistributedCallable distCallable = new MWTestDistributedCallable(
          this, sourceCacheName, destinationCacheName);

      org.infinispan.distexec.DefaultExecutorService infinispanExecutorService;
      DistributedExecutorService des = new DefaultExecutorService(sourceCache);
      
//      final IGridManager gridManager = GridManagerFactory.getInstance()
//          .getDefaultGridManager(sourceCache.getCacheManager());
//      final IDistributedExecutorService des = gridManager
//          .getDistributedExecutorService(TestHelper.getInstance().getTreeCache(sourceCache));
      Object[] keys = sourceChildPaths.toArray();
      final List<Future<Long>> futureResults = des.submitEverywhere(distCallable, keys);

      // Better have futureResults or we're in trouble!
      if (futureResults == null)
      {
        throw new Exception("futureResults is null");
      }

      // Confirm that the number of paths processed is the
      // number it's supposed to be
      long totalChildPathsProcessed = 0;
      for (final Future<Long> futureResult : futureResults)
      {
        totalChildPathsProcessed += futureResult.get().longValue();
      }

      if (totalChildPathsProcessed != NUM_SOURCE_NODES)
      {
        throw new Exception("totalChildPathsProcessed should be "
            + NUM_SOURCE_NODES + " but instead is " + totalChildPathsProcessed);
      }

      // Confirm the keys and values on the root node
      final TreeCache<Object, Object> destinationTreeCache = TestHelper.getInstance().getTreeCache(destinationCache);
      final Node<Object, Object> destinationRootNode = destinationTreeCache.getRoot();
      for (final Fqn sourceChildPath : sourceChildPaths)
      {
        final String key = sourceChildPath.toString().substring(1);
        Object value = destinationRootNode.get(key);
        if (value == null)
        {
          throw new Exception(destinationCache.getName()+" Destination root node lacked a key for "
              + key);
        }

        if (!value.equals(key))
        {
          throw new Exception(
              destinationCache.getName()+" Destination root node had the wrong value for key "
                  + key + ", value was " + value.toString());
        }
        
        log.info(destinationCache.getName()+" Found good key " + key + " in destination root node");
      }
    } catch (final Exception ex)
    {
      throw new Exception(ex);
    }
  }

}

class MWTestDistributedCallable implements
    DistributedCallable<Object, Object, Long>, Serializable
{
  private static final long         serialVersionUID = -3192895477405026051L;
  private final TestMultiWriter     mwTest;
  private final String              sourceCacheName;
  private final String              destinationCacheName;
  private final List<Fqn> pathsToProcess;

  public MWTestDistributedCallable(final TestMultiWriter mwTest,
      final String sourceCacheName, final String destinationCacheName)
  {
    this.mwTest = mwTest;
    this.sourceCacheName = sourceCacheName;
    this.destinationCacheName = destinationCacheName;
    this.pathsToProcess = new ArrayList<Fqn>();
  }

  @Override
  public Long call() throws Exception
  {
    long numPathsProcessed = 0;

    if (pathsToProcess.size() > 0)
    {
      final ExecutorService pool = TestHelper.getInstance().getExecutorService();
      final List<Future<Long>> futureResults = new ArrayList<Future<Long>>();
      for (final Fqn pathToProcess : pathsToProcess)
      {
        final Callable<Long> callable = new MWTestCallable(mwTest,
            sourceCacheName, destinationCacheName, pathToProcess);
        final Future<Long> futureResult = pool.submit(callable);
        futureResults.add(futureResult);
      }

      for (final Future<Long> futureResult : futureResults)
      {
        numPathsProcessed += futureResult.get().longValue();
      }
    }

    return Long.valueOf(numPathsProcessed);
  }

//  @Override
//  public void setEnvironment(final ICache<Object, Object> cache,
//      final Set<Object> inputKeys)
//  {
//    if (cache.getTreeCache() != null)
//    {
//      setEnvironment(cache.getTreeCache(), inputKeys);
//    }
//  }
//
//  @Override
//  public void setEnvironment(final ITreeCache<Object, Object> treeCache,
//      final Set<Object> inputKeys)
//  {
//    pathsToProcess.clear();
//    for (final Object inputKey : inputKeys)
//    {
//      final Fqn path = (Fqn) inputKey;
//      pathsToProcess.add(path);
//    }
//  }

  @Override
  public void setEnvironment(Cache<Object, Object> cache, Set<Object> inputKeys)
  {
    pathsToProcess.clear();
    for (final Object inputKey : inputKeys)
    {
      final Fqn path = (Fqn) inputKey;
      pathsToProcess.add(path);
    }
    
  }
}

class MWTestCallable implements Callable<Long>, Serializable
{
  private static final long     serialVersionUID = -1572063052116806145L;
  private static final Logger   log              = LoggerFactory
                                                     .getLogger(TestMultiWriter.class);

  private final TestMultiWriter mwTest;
  private final String          sourceCacheName;
  private final String          destinationCacheName;
  private final Fqn   pathToProcess;

  protected MWTestCallable(final TestMultiWriter mwTest,
      final String sourceCacheName, final String destinationCacheName,
      final Fqn pathToProcess)
  {
    this.mwTest = mwTest;
    this.sourceCacheName = sourceCacheName;
    this.destinationCacheName = destinationCacheName;
    this.pathToProcess = pathToProcess;
  }

  @Override
  public Long call() throws Exception
  {
    final CacheManager cacheManager = TestHelper.getInstance().getCacheManager();

    final Cache<Object, Object> sourceCache = cacheManager
        .getCache(sourceCacheName);
    final Cache<Object, Object> destinationCache = cacheManager
        .getCache(destinationCacheName);

    final TreeCache<Object, Object> sourceTreeCache = TestHelper.getInstance().getTreeCache(sourceCache);
        
    TreeCache<Object, Object> destinationTreeCache = TestHelper.getInstance().getTreeCache(destinationCache);
        

    final Node<Object, Object> sourceTreeNode = sourceTreeCache.getNode(pathToProcess);
    final Node<Object, Object> destinationTreeNode = destinationTreeCache
        .getRoot();

    String valueToPutOnRootNode = sourceTreeNode.getFqn().toString();

    if (!valueToPutOnRootNode.equals(pathToProcess.toString()))
    {
      throw new Exception("The node names do not match: "
          + valueToPutOnRootNode + ", " + pathToProcess.toString());
    }

    if(valueToPutOnRootNode.startsWith("/"))
    {
      valueToPutOnRootNode = valueToPutOnRootNode.substring(1);
    }
    
    destinationTreeNode.put(valueToPutOnRootNode, valueToPutOnRootNode);
    log.info("Put key and value '" + valueToPutOnRootNode + "' on cache " + destinationCacheName + " root node");
    
    return Long.valueOf(1);
  }
}

class MWTestCacheSignature implements ITreeCacheSignatureTest
{
  private static final long serialVersionUID = 6982205318901622122L;

  private final String      name;

  protected MWTestCacheSignature(final String name)
  {
    this.name = name;
  }

  protected String getName()
  {
    return name;
  }

  public boolean equals(final Object o)
  {
    if (o == this)
    {
      return true;
    }

    if (!(o instanceof MWTestCacheSignature))
    {
      return false;
    }

    final MWTestCacheSignature sig = (MWTestCacheSignature) o;
    return name.equals(sig.getName());
  }

}
